@@ -352,6 +352,7 @@ int osmo_ss7_as_del_asp(struct osmo_ss7_as *as, const char *asp_name);
 void osmo_ss7_as_destroy(struct osmo_ss7_as *as);
 bool osmo_ss7_as_has_asp(const struct osmo_ss7_as *as,
 			 const struct osmo_ss7_asp *asp);
+bool osmo_ss7_as_down(const struct osmo_ss7_as *as);
 bool osmo_ss7_as_active(const struct osmo_ss7_as *as);
 bool osmo_ss7_as_tmode_compatible_xua(struct osmo_ss7_as *as, uint32_t m3ua_tmt);
 void osmo_ss7_asp_disconnect(struct osmo_ss7_asp *asp);
@@ -365,6 +366,8 @@ struct osmo_ss7_asp_peer {
 	char *host[OSMO_SOCK_MAX_ADDRS];
 	size_t host_cnt;
 	uint16_t port;
+	/* index in "hosts" array marking the SCTP Primary Address, -1 if no explicit Primary Address set */
+	int idx_primary;
 };
 
 enum osmo_ss7_asp_admin_state {
@@ -430,11 +433,29 @@ struct osmo_ss7_asp {
 		bool is_server;
 		enum osmo_ss7_asp_role role;
 		bool role_set_by_vty;
+		bool trans_role_set_by_vty;
 
 		struct osmo_ss7_asp_peer local;
 		struct osmo_ss7_asp_peer remote;
 		uint8_t qos_class;
 		uint32_t quirks;
+
+		/* T_defs used by the default_lm: */
+		struct osmo_tdef *T_defs_lm;
+
+		struct {
+			bool num_ostreams_present;
+			bool max_instreams_present;
+			bool max_attempts_present;
+			bool max_init_timeo_present;
+			uint16_t num_ostreams_value;
+			uint16_t max_instreams_value;
+			uint16_t max_attempts_value;
+			uint16_t max_init_timeo_value; /* ms */
+		} sctp_init;
+
+		/*! The underlaying transport protocol (one of IPPROTO_*) */
+		int trans_proto;
 	} cfg;
 };
 
@@ -446,29 +467,48 @@ struct osmo_ss7_asp {
 #define OSMO_SS7_ASP_QUIRK_SNM_INACTIVE		0x00000004
 
 int osmo_ss7_asp_peer_snprintf(char* buf, size_t buf_len, struct osmo_ss7_asp_peer *peer);
+void osmo_ss7_asp_peer_init(struct osmo_ss7_asp_peer *peer);
 int osmo_ss7_asp_peer_set_hosts(struct osmo_ss7_asp_peer *peer, void *talloc_ctx,
-				const char* const* hosts, size_t host_cnt);
+				const char *const*hosts, size_t host_cnt);
+int osmo_ss7_asp_peer_set_hosts2(struct osmo_ss7_asp_peer *peer, void *talloc_ctx,
+				const char *const*hosts, size_t host_cnt, int idx_primary);
 int osmo_ss7_asp_peer_add_host(struct osmo_ss7_asp_peer *peer, void *talloc_ctx, const char *host);
+int osmo_ss7_asp_peer_add_host2(struct osmo_ss7_asp_peer *peer, void *talloc_ctx, const char *host, bool is_primary_addr);
+int osmo_ss7_asp_peer_del_host(struct osmo_ss7_asp_peer *peer, const char *host);
+
 
 struct osmo_ss7_asp *
 osmo_ss7_asp_find_by_name(struct osmo_ss7_instance *inst, const char *name);
-struct osmo_ss7_asp
-*osmo_ss7_asp_find_by_proto(struct osmo_ss7_as *as,
-			    enum osmo_ss7_asp_protocol proto);
+struct osmo_ss7_asp *
+osmo_ss7_asp_find_by_proto(struct osmo_ss7_as *as,
+			   enum osmo_ss7_asp_protocol proto);
 struct osmo_ss7_asp *
 osmo_ss7_asp_find(struct osmo_ss7_instance *inst, const char *name,
 		  uint16_t remote_port, uint16_t local_port,
-		  enum osmo_ss7_asp_protocol proto);
+		  enum osmo_ss7_asp_protocol proto)
+	OSMO_DEPRECATED("Use osmo_ss7_asp_find2() instead");
+struct osmo_ss7_asp *
+osmo_ss7_asp_find2(struct osmo_ss7_instance *inst, const char *name,
+		   uint16_t remote_port, uint16_t local_port,
+		   int trans_proto, enum osmo_ss7_asp_protocol proto);
 struct osmo_ss7_asp *
 osmo_ss7_asp_find_or_create(struct osmo_ss7_instance *inst, const char *name,
 			    uint16_t remote_port, uint16_t local_port,
-			    enum osmo_ss7_asp_protocol proto);
+			    enum osmo_ss7_asp_protocol proto)
+	OSMO_DEPRECATED("Use osmo_ss7_asp_find_or_create2() instead");
+struct osmo_ss7_asp *
+osmo_ss7_asp_find_or_create2(struct osmo_ss7_instance *inst, const char *name,
+			     uint16_t remote_port, uint16_t local_port,
+			     int trans_proto, enum osmo_ss7_asp_protocol proto);
 void osmo_ss7_asp_destroy(struct osmo_ss7_asp *asp);
 int osmo_ss7_asp_send(struct osmo_ss7_asp *asp, struct msgb *msg);
 int osmo_ss7_asp_restart(struct osmo_ss7_asp *asp);
 int osmo_ss7_asp_use_default_lm(struct osmo_ss7_asp *asp, int log_level);
 bool osmo_ss7_asp_active(const struct osmo_ss7_asp *asp);
 int osmo_ss7_asp_get_log_subsys(const struct osmo_ss7_asp *asp);
+const char *osmo_ss7_asp_get_name(const struct osmo_ss7_asp *asp);
+enum osmo_ss7_asp_protocol osmo_ss7_asp_get_proto(const struct osmo_ss7_asp *asp);
+int osmo_ss7_asp_get_trans_proto(const struct osmo_ss7_asp *asp);
 
 /*! Weak function to handle payload for unknown/unsupported PPID or IPA StreamID.
  *  This function can be overridden by application code to implement whatever handling
@@ -511,16 +551,38 @@ struct osmo_xua_server {
 		bool accept_dyn_reg;
 		struct osmo_ss7_asp_peer local;
 		enum osmo_ss7_asp_protocol proto;
+		struct {
+			bool num_ostreams_present;
+			bool max_instreams_present;
+			uint16_t num_ostreams_value;
+			uint16_t max_instreams_value;
+		} sctp_init;
+
+		/*! The underlaying transport protocol (one of IPPROTO_*) */
+		int trans_proto;
 	} cfg;
 };
 
 struct osmo_xua_server *
-osmo_ss7_xua_server_find(struct osmo_ss7_instance *inst, enum osmo_ss7_asp_protocol proto,
-			 uint16_t local_port);
+osmo_ss7_xua_server_find(struct osmo_ss7_instance *inst,
+			 enum osmo_ss7_asp_protocol proto,
+			 uint16_t local_port)
+	OSMO_DEPRECATED("Use osmo_ss7_xua_server_find2() instead");
+struct osmo_xua_server *
+osmo_ss7_xua_server_find2(struct osmo_ss7_instance *inst,
+			  int trans_proto,
+			  enum osmo_ss7_asp_protocol proto,
+			  uint16_t local_port);
 
 struct osmo_xua_server *
-osmo_ss7_xua_server_create(struct osmo_ss7_instance *inst, enum osmo_ss7_asp_protocol proto,
-			   uint16_t local_port, const char *local_host);
+osmo_ss7_xua_server_create(struct osmo_ss7_instance *inst,
+			   enum osmo_ss7_asp_protocol proto,
+			   uint16_t local_port, const char *local_host)
+	OSMO_DEPRECATED("Use osmo_ss7_xua_server_create2() instead");
+struct osmo_xua_server *
+osmo_ss7_xua_server_create2(struct osmo_ss7_instance *inst,
+			    int trans_proto, enum osmo_ss7_asp_protocol proto,
+			    uint16_t local_port, const char *local_host);
 
 int
 osmo_ss7_xua_server_bind(struct osmo_xua_server *xs);
@@ -530,6 +592,7 @@ osmo_ss7_xua_server_set_local_host(struct osmo_xua_server *xs, const char *local
 int
 osmo_ss7_xua_server_set_local_hosts(struct osmo_xua_server *xs, const char **local_hosts, size_t local_host_cnt);
 int osmo_ss7_xua_server_add_local_host(struct osmo_xua_server *xs, const char *local_host);
+int osmo_ss7_xua_server_del_local_host(struct osmo_xua_server *xs, const char *local_host);
 void osmo_ss7_xua_server_destroy(struct osmo_xua_server *xs);
 
 struct osmo_sccp_instance *
@@ -564,6 +627,8 @@ osmo_sccp_simple_server_add_clnt(struct osmo_sccp_instance *inst,
 				 int local_port, int remote_port,
 				 const char *remote_ip);
 
+void osmo_sccp_set_max_optional_data(struct osmo_sccp_instance *inst, int val);
+
 enum osmo_ss7_as_traffic_mode osmo_ss7_tmode_from_xua(uint32_t in);
 int osmo_ss7_tmode_to_xua(enum osmo_ss7_as_traffic_mode tmod);
 
