@@ -69,14 +69,6 @@ const struct value_string mtp_unavail_cause_vals[] = {
 	{ 0, NULL }
 };
 
-struct value_string osmo_ss7_as_traffic_mode_vals[] = {
-	{ OSMO_SS7_AS_TMOD_BCAST,	"broadcast" },
-	{ OSMO_SS7_AS_TMOD_LOADSHARE,	"loadshare" },
-	{ OSMO_SS7_AS_TMOD_ROUNDROBIN,	"round-robin" },
-	{ OSMO_SS7_AS_TMOD_OVERRIDE,	"override" },
-	{ 0, NULL }
-};
-
 int osmo_ss7_find_free_rctx(struct osmo_ss7_instance *inst)
 {
 	int32_t rctx;
@@ -88,7 +80,7 @@ int osmo_ss7_find_free_rctx(struct osmo_ss7_instance *inst)
 	return -1;
 }
 
-static uint32_t find_free_l_rk_id(struct osmo_ss7_instance *inst)
+uint32_t ss7_find_free_l_rk_id(struct osmo_ss7_instance *inst)
 {
 	uint32_t l_rk_id;
 
@@ -855,19 +847,6 @@ const char *osmo_ss7_route_print(const struct osmo_ss7_route *rt)
  * SS7 Application Server
  ***********************************************************************/
 
-static const struct rate_ctr_desc ss7_as_rcd[] = {
-	[SS7_AS_CTR_RX_MSU_TOTAL] = { "rx:msu:total", "Total number of MSU received" },
-	[SS7_AS_CTR_TX_MSU_TOTAL] = { "tx:msu:total", "Total number of MSU transmitted" },
-};
-
-static const struct rate_ctr_group_desc ss7_as_rcgd = {
-	.group_name_prefix = "sigtran_as",
-	.group_description = "SIGTRAN Application Server",
-	.num_ctr = ARRAY_SIZE(ss7_as_rcd),
-	.ctr_desc = ss7_as_rcd,
-};
-static unsigned int g_ss7_as_rcg_idx;
-
 /*! \brief Find Application Server by given name
  *  \param[in] inst SS7 Instance on which we operate
  *  \param[in] name Name of AS
@@ -936,18 +915,17 @@ struct osmo_ss7_as *osmo_ss7_as_find_by_proto(struct osmo_ss7_instance *inst,
 	/* Loop through the list with AS and try to find one where the proto
 	   matches up */
 	llist_for_each_entry(as, &inst->as_list, list) {
-		if (as->cfg.proto == proto) {
+		if (as->cfg.proto != proto)
+			continue;
 
-			/* Put down the first AS that matches the proto, just in
-			 * case we will not find any matching ASP */
-			if (!as_without_asp)
-				as_without_asp = as;
+		/* Put down the first AS that matches the proto, just in
+		 * case we will not find any matching ASP */
+		if (!as_without_asp)
+			as_without_asp = as;
 
-			/* Check if the candicate we have here has any suitable
-			 * ASP */
-			if (osmo_ss7_asp_find_by_proto(as, proto))
-				return as;
-		}
+		/* Check if the candicate we have here has any suitable ASP */
+		if (osmo_ss7_asp_find_by_proto(as, proto))
+			return as;
 	}
 
 	/* Return with the second best find, if there is any */
@@ -972,147 +950,15 @@ osmo_ss7_as_find_or_create(struct osmo_ss7_instance *inst, const char *name,
 		return NULL;
 
 	if (!as) {
-		as = talloc_zero(inst, struct osmo_ss7_as);
+		as = ss7_as_alloc(inst, name, proto);
 		if (!as)
 			return NULL;
-		as->ctrg = rate_ctr_group_alloc(as, &ss7_as_rcgd, g_ss7_as_rcg_idx++);
-		if (!as->ctrg) {
-			talloc_free(as);
-			return NULL;
-		}
-		rate_ctr_group_set_name(as->ctrg, name);
-		as->inst = inst;
-		as->cfg.name = talloc_strdup(as, name);
-		as->cfg.proto = proto;
-		as->cfg.mode = OSMO_SS7_AS_TMOD_OVERRIDE;
-		as->cfg.recovery_timeout_msec = 2000;
-		as->cfg.routing_key.l_rk_id = find_free_l_rk_id(inst);
-		as->fi = xua_as_fsm_start(as, LOGL_DEBUG);
-		llist_add_tail(&as->list, &inst->as_list);
 		LOGPAS(as, DLSS7, LOGL_INFO, "Created AS\n");
 	}
 
 	return as;
 }
 
-/*! \brief Add given ASP to given AS
- *  \param[in] as Application Server to which \ref asp is added
- *  \param[in] asp Application Server Process to be added to \ref as
- *  \returns 0 on success; negative in case of error */
-int osmo_ss7_as_add_asp(struct osmo_ss7_as *as, const char *asp_name)
-{
-	struct osmo_ss7_asp *asp;
-	unsigned int i;
-
-	OSMO_ASSERT(ss7_initialized);
-	asp = osmo_ss7_asp_find_by_name(as->inst, asp_name);
-	if (!asp)
-		return -ENODEV;
-
-	LOGPAS(as, DLSS7, LOGL_INFO, "Adding ASP %s to AS\n", asp->cfg.name);
-
-	if (osmo_ss7_as_has_asp(as, asp))
-		return 0;
-
-	for (i = 0; i < ARRAY_SIZE(as->cfg.asps); i++) {
-		if (!as->cfg.asps[i]) {
-			as->cfg.asps[i] = asp;
-			return 0;
-		}
-	}
-
-	return -ENOSPC;
-}
-
-/*! \brief Delete given ASP from given AS
- *  \param[in] as Application Server from which \ref asp is deleted
- *  \param[in] asp Application Server Process to delete from \ref as
- *  \returns 0 on success; negative in case of error */
-int osmo_ss7_as_del_asp(struct osmo_ss7_as *as, const char *asp_name)
-{
-	struct osmo_ss7_asp *asp;
-	unsigned int i;
-
-	OSMO_ASSERT(ss7_initialized);
-	asp = osmo_ss7_asp_find_by_name(as->inst, asp_name);
-	if (!asp)
-		return -ENODEV;
-
-	LOGPAS(as, DLSS7, LOGL_INFO, "Removing ASP %s from AS\n", asp->cfg.name);
-
-	for (i = 0; i < ARRAY_SIZE(as->cfg.asps); i++) {
-		if (as->cfg.asps[i] == asp) {
-			as->cfg.asps[i] = NULL;
-			return 0;
-		}
-	}
-
-	return -EINVAL;
-}
-
-/*! \brief Destroy given Application Server
- *  \param[in] as Application Server to destroy */
-void osmo_ss7_as_destroy(struct osmo_ss7_as *as)
-{
-	struct osmo_ss7_route *rt, *rt2;
-
-	OSMO_ASSERT(ss7_initialized);
-	LOGPAS(as, DLSS7, LOGL_INFO, "Destroying AS\n");
-
-	if (as->fi)
-		osmo_fsm_inst_term(as->fi, OSMO_FSM_TERM_REQUEST, NULL);
-
-	/* find any routes pointing to this AS and remove them */
-	llist_for_each_entry_safe(rt, rt2, &as->inst->rtable_system->routes, list) {
-		if (rt->dest.as == as)
-			osmo_ss7_route_destroy(rt);
-	}
-
-	as->inst = NULL;
-	llist_del(&as->list);
-	rate_ctr_group_free(as->ctrg);
-	talloc_free(as);
-}
-
-/*! \brief Determine if given AS contains ASP
- *  \param[in] as Application Server in which to look for \ref asp
- *  \param[in] asp Application Server Process to look for in \ref as
- *  \returns true in case \ref asp is part of \ref as; false otherwise */
-bool osmo_ss7_as_has_asp(const struct osmo_ss7_as *as,
-			 const struct osmo_ss7_asp *asp)
-{
-	unsigned int i;
-
-	OSMO_ASSERT(ss7_initialized);
-	for (i = 0; i < ARRAY_SIZE(as->cfg.asps); i++) {
-		if (as->cfg.asps[i] == asp)
-			return true;
-	}
-	return false;
-}
-
-/*! Determine if given AS is in the active state.
- *  \param[in] as Application Server.
- *  \returns true in case as is active; false otherwise. */
-bool osmo_ss7_as_active(const struct osmo_ss7_as *as)
-{
-	if (!as->fi)
-		return false;
-	return as->fi->state == XUA_AS_S_ACTIVE;
-}
-
-/*! Determine if given AS is in the down state.
- *  \param[in] as Application Server.
- *  \returns true in case as is down; false otherwise. */
-bool osmo_ss7_as_down(const struct osmo_ss7_as *as)
-{
-	OSMO_ASSERT(as);
-
-	if (!as->fi)
-		return true;
-	return as->fi->state == XUA_AS_S_DOWN;
-}
-
 bool ss7_ipv6_sctp_supported(const char *host, bool bind)
 {
 	int rc;
@@ -1152,12 +998,13 @@ osmo_ss7_asp_find_by_name(struct osmo_ss7_instance *inst, const char *name)
 	return NULL;
 }
 
-/*! \brief Find an ASP that matches the given protocol.
+/*! \brief Find an ASP that matches the given ASP protocol (xUA variant).
  *  \param[in] as Application Server in which to look for \ref asp
+ *  \param[in] proto ASP protocol (xUA variant) to match
  *  \returns SS7 ASP in case a matching one is found; NULL otherwise */
-struct osmo_ss7_asp
-*osmo_ss7_asp_find_by_proto(struct osmo_ss7_as *as,
-			    enum osmo_ss7_asp_protocol proto)
+struct osmo_ss7_asp *
+osmo_ss7_asp_find_by_proto(struct osmo_ss7_as *as,
+			   enum osmo_ss7_asp_protocol proto)
 {
 	unsigned int i;
 
@@ -1170,9 +1017,9 @@ struct osmo_ss7_asp
 }
 
 struct osmo_ss7_asp *
-osmo_ss7_asp_find(struct osmo_ss7_instance *inst, const char *name,
-		  uint16_t remote_port, uint16_t local_port,
-		  enum osmo_ss7_asp_protocol proto)
+osmo_ss7_asp_find2(struct osmo_ss7_instance *inst, const char *name,
+		   uint16_t remote_port, uint16_t local_port,
+		   int trans_proto, enum osmo_ss7_asp_protocol proto)
 {
 	struct osmo_ss7_asp *asp;
 
@@ -1181,47 +1028,107 @@ osmo_ss7_asp_find(struct osmo_ss7_instance *inst, const char *name,
 	if (!asp)
 		return NULL;
 
-	if ((asp->cfg.remote.port != remote_port || asp->cfg.local.port != local_port || asp->cfg.proto != proto))
+	if (asp->cfg.remote.port != remote_port || asp->cfg.local.port != local_port)
+		return NULL;
+	if (asp->cfg.trans_proto != trans_proto)
+		return NULL;
+	if (asp->cfg.proto != proto)
 		return NULL;
 
 	return asp;
 }
 
+struct osmo_ss7_asp *
+osmo_ss7_asp_find(struct osmo_ss7_instance *inst, const char *name,
+		  uint16_t remote_port, uint16_t local_port,
+		  enum osmo_ss7_asp_protocol proto)
+{
+	const int trans_proto = ss7_default_trans_proto_for_asp_proto(proto);
+
+	return osmo_ss7_asp_find2(inst, name,
+				  remote_port, local_port,
+				  trans_proto, proto);
+}
+
+struct osmo_ss7_asp *
+osmo_ss7_asp_find_or_create2(struct osmo_ss7_instance *inst, const char *name,
+			     uint16_t remote_port, uint16_t local_port,
+			     int trans_proto, enum osmo_ss7_asp_protocol proto)
+{
+	struct osmo_ss7_asp *asp;
+
+	OSMO_ASSERT(ss7_initialized);
+	asp = osmo_ss7_asp_find_by_name(inst, name);
+	if (asp) {
+		if (asp->cfg.remote.port != remote_port || asp->cfg.local.port != local_port)
+			return NULL;
+		if (asp->cfg.trans_proto != trans_proto)
+			return NULL;
+		if (asp->cfg.proto != proto)
+			return NULL;
+		return asp;
+	}
+
+	return ss7_asp_alloc(inst, name, remote_port, local_port, trans_proto, proto);
+}
+
 struct osmo_ss7_asp *
 osmo_ss7_asp_find_or_create(struct osmo_ss7_instance *inst, const char *name,
 			    uint16_t remote_port, uint16_t local_port,
 			    enum osmo_ss7_asp_protocol proto)
 {
-	struct osmo_ss7_asp *asp;
+	const int trans_proto = ss7_default_trans_proto_for_asp_proto(proto);
 
-	OSMO_ASSERT(ss7_initialized);
-	asp = osmo_ss7_asp_find_by_name(inst, name);
-	if (asp) {
-		if (asp->cfg.remote.port != remote_port ||
-		    asp->cfg.local.port != local_port ||
-		    asp->cfg.proto != proto)
-			return NULL;
-		return asp;
-	}
-
-	return ss7_asp_alloc(inst, name, remote_port, local_port, proto);
+	return osmo_ss7_asp_find_or_create2(inst, name,
+					    remote_port, local_port,
+					    trans_proto, proto);
 }
 
+/*! \brief find an xUA server with the given parameters
+ *  \param[in] inst SS7 Instance on which we operate
+ *  \param[in] trans_proto transport protocol in use (one of IPPROTO_*)
+ *  \param[in] proto protocol (xUA variant) in use
+ *  \param[in] local_port local port of the server
+ *  \returns \ref osmo_xua_server or NULL (not found)
+ */
 struct osmo_xua_server *
-osmo_ss7_xua_server_find(struct osmo_ss7_instance *inst, enum osmo_ss7_asp_protocol proto,
-			 uint16_t local_port)
+osmo_ss7_xua_server_find2(struct osmo_ss7_instance *inst,
+			  int trans_proto,
+			  enum osmo_ss7_asp_protocol proto,
+			  uint16_t local_port)
 {
 	struct osmo_xua_server *xs;
 
 	OSMO_ASSERT(ss7_initialized);
 	llist_for_each_entry(xs, &inst->xua_servers, list) {
-		if (proto == xs->cfg.proto &&
-		    local_port == xs->cfg.local.port)
-			return xs;
+		if (trans_proto != xs->cfg.trans_proto)
+			continue;
+		if (proto != xs->cfg.proto)
+			continue;
+		if (local_port != xs->cfg.local.port)
+			continue;
+		return xs;
 	}
+
 	return NULL;
 }
 
+/*! \brief find an xUA server with the given parameters
+ *  \param[in] inst SS7 Instance on which we operate
+ *  \param[in] proto protocol (xUA variant) in use
+ *  \param[in] local_port local port of the server
+ *  \returns \ref osmo_xua_server or NULL (not found)
+ */
+struct osmo_xua_server *
+osmo_ss7_xua_server_find(struct osmo_ss7_instance *inst,
+			 enum osmo_ss7_asp_protocol proto,
+			 uint16_t local_port)
+{
+	const int trans_proto = ss7_default_trans_proto_for_asp_proto(proto);
+
+	return osmo_ss7_xua_server_find2(inst, trans_proto, proto, local_port);
+}
+
 bool osmo_ss7_pc_is_local(struct osmo_ss7_instance *inst, uint32_t pc)
 {
 	OSMO_ASSERT(ss7_initialized);
