@@ -6,6 +6,8 @@
  *
  * All Rights Reserved
  *
+ * SPDX-License-Identifier: GPL-2.0+
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -16,20 +18,18 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
+#include <errno.h>
 #include <string.h>
 
 #include <osmocom/core/msgb.h>
 #include <osmocom/core/talloc.h>
 #include <osmocom/core/logging.h>
+#include <osmocom/core/endian.h>
 #include <osmocom/gsm/tlv.h>
-
-#include <sccp/sccp.h>
+#include <osmocom/sccp/sccp_types.h>
+#include <osmocom/sccp/sccp.h>
 
 // Unassigned debug area
 static int DSCCP = 0;
@@ -227,6 +227,12 @@ int _sccp_parse_connection_request(struct msgb *msgb, struct sccp_parse_result *
 	}
 
 	if (optional_data.data_len != 0) {
+		if (optional_data.data_len > SCCP_MAX_OPTIONAL_DATA) {
+			LOGP(DSCCP, LOGL_ERROR,
+				 "optional data has length %u exceeding max of %u according to ITU-T Rec. Q.713 §4.2\n",
+				 optional_data.data_len, SCCP_MAX_OPTIONAL_DATA);
+			return -EMSGSIZE;
+		}
 		msgb->l3h = &msgb->l2h[optional_data.data_start];
 		result->data_len = optional_data.data_len;
 	} else {
@@ -261,6 +267,12 @@ int _sccp_parse_connection_released(struct msgb *msgb, struct sccp_parse_result
 	result->destination_local_reference = &rls->destination_local_reference;
 
 	if (optional_data.data_len != 0) {
+		if (optional_data.data_len > SCCP_MAX_OPTIONAL_DATA) {
+			LOGP(DSCCP, LOGL_ERROR,
+				 "optional data has length %u exceeding max of %u according to ITU-T Rec. Q.713 §4.5\n",
+				 optional_data.data_len, SCCP_MAX_OPTIONAL_DATA);
+			return -EMSGSIZE;
+		}
 		msgb->l3h = &msgb->l2h[optional_data.data_start];
 		result->data_len = optional_data.data_len;
 	} else {
@@ -298,6 +310,12 @@ int _sccp_parse_connection_refused(struct msgb *msgb, struct sccp_parse_result *
 
 	/* optional data */
 	if (optional_data.data_len != 0) {
+		if (optional_data.data_len > SCCP_MAX_OPTIONAL_DATA) {
+			LOGP(DSCCP, LOGL_ERROR,
+				 "optional data has length %u exceeding max of %u according to ITU-T Rec. Q.713 §4.4\n",
+				 optional_data.data_len, SCCP_MAX_OPTIONAL_DATA);
+			return -EMSGSIZE;
+		}
 		msgb->l3h = &msgb->l2h[optional_data.data_start];
 		result->data_len = optional_data.data_len;
 	} else {
@@ -335,6 +353,12 @@ int _sccp_parse_connection_confirm(struct msgb *msgb, struct sccp_parse_result *
 	}
 
 	if (optional_data.data_len != 0) {
+		if (optional_data.data_len > SCCP_MAX_OPTIONAL_DATA) {
+			LOGP(DSCCP, LOGL_ERROR,
+				 "optional data has length %u exceeding max of %u according to ITU-T Rec. Q.713 §4.3\n",
+				 optional_data.data_len, SCCP_MAX_OPTIONAL_DATA);
+			return -EMSGSIZE;
+		}
 		msgb->l3h = &msgb->l2h[optional_data.data_start];
 		result->data_len = optional_data.data_len;
 	} else {
@@ -404,37 +428,46 @@ int _sccp_parse_connection_dt1(struct msgb *msgb, struct sccp_parse_result *resu
 	return 0;
 }
 
-int _sccp_parse_udt(struct msgb *msgb, struct sccp_parse_result *result)
+struct udt_offsets {
+	uint32_t	header_size;
+	uint32_t	called_offset;
+	uint32_t	calling_offset;
+	uint32_t	data_offset;
+};
+
+static int _sccp_parse_unitdata(struct msgb *msgb, struct sccp_parse_result *result,
+				const struct udt_offsets *offs)
 {
-	static const uint32_t header_size = sizeof(struct sccp_data_unitdata);
-	static const uint32_t called_offset = offsetof(struct sccp_data_unitdata, variable_called);
-	static const uint32_t calling_offset = offsetof(struct sccp_data_unitdata, variable_calling);
-	static const uint32_t data_offset = offsetof(struct sccp_data_unitdata, variable_data);
+	uint8_t		variable_called;
+	uint8_t		variable_calling;
+	uint8_t		variable_data;
 
-	struct sccp_data_unitdata *udt = (struct sccp_data_unitdata *)msgb->l2h;
-
-	if (msgb_l2len(msgb) < header_size) {
+	if (msgb_l2len(msgb) < offs->header_size) {
 		LOGP(DSCCP, LOGL_ERROR, "msgb < header_size %u %u\n",
-		        msgb_l2len(msgb), header_size);
+		        msgb_l2len(msgb), offs->header_size);
 		return -1;
 	}
 
+	variable_called = msgb->l2h[offs->called_offset];
+	variable_calling = msgb->l2h[offs->calling_offset];
+	variable_data = msgb->l2h[offs->data_offset];
+
 	/* copy out the calling and called address. Add the off */
-	if (copy_address(&result->called, called_offset + udt->variable_called, msgb) != 0)
+	if (copy_address(&result->called, offs->called_offset + variable_called, msgb) != 0)
 		return -1;
 
-	if (copy_address(&result->calling, calling_offset + udt->variable_calling, msgb) != 0)
+	if (copy_address(&result->calling, offs->calling_offset + variable_calling, msgb) != 0)
 		return -1;
 
 	/* we don't have enough size for the data */
-	if (msgb_l2len(msgb) < data_offset + udt->variable_data + 1) {
+	if (msgb_l2len(msgb) < offs->data_offset + variable_data + 1) {
 		LOGP(DSCCP, LOGL_ERROR, "msgb < header + offset %u %u %u\n",
-			msgb_l2len(msgb), header_size, udt->variable_data);
+			msgb_l2len(msgb), offs->header_size, variable_data);
 		return -1;
 	}
 
 
-	msgb->l3h = &udt->data[udt->variable_data];
+	msgb->l3h = &msgb->l2h[offs->data_offset + variable_data + 1];
 	result->data_len = msgb_l3len(msgb);
 
 	if (msgb_l3len(msgb) <  msgb->l3h[-1]) {
@@ -446,6 +479,54 @@ int _sccp_parse_udt(struct msgb *msgb, struct sccp_parse_result *result)
 	return 0;
 }
 
+int _sccp_parse_udt(struct msgb *msgb, struct sccp_parse_result *result)
+{
+	static const struct udt_offsets offsets = {
+		.header_size = sizeof(struct sccp_data_unitdata),
+		.called_offset = offsetof(struct sccp_data_unitdata, variable_called),
+		.calling_offset = offsetof(struct sccp_data_unitdata, variable_calling),
+		.data_offset = offsetof(struct sccp_data_unitdata, variable_data),
+	};
+
+	return _sccp_parse_unitdata(msgb, result, &offsets);
+}
+
+int _sccp_parse_udts(struct msgb *msgb, struct sccp_parse_result *result)
+{
+	static const struct udt_offsets offsets = {
+		.header_size = sizeof(struct sccp_data_unitdata_service),
+		.called_offset = offsetof(struct sccp_data_unitdata_service, variable_called),
+		.calling_offset = offsetof(struct sccp_data_unitdata_service, variable_calling),
+		.data_offset = offsetof(struct sccp_data_unitdata_service, variable_data),
+	};
+
+	return _sccp_parse_unitdata(msgb, result, &offsets);
+}
+
+static int _sccp_parse_xudt(struct msgb *msgb, struct sccp_parse_result *result)
+{
+	static const struct udt_offsets offsets = {
+		.header_size = sizeof(struct sccp_data_ext_unitdata),
+		.called_offset = offsetof(struct sccp_data_ext_unitdata, variable_called),
+		.calling_offset = offsetof(struct sccp_data_ext_unitdata, variable_calling),
+		.data_offset = offsetof(struct sccp_data_ext_unitdata, variable_data),
+	};
+
+	return _sccp_parse_unitdata(msgb, result, &offsets);
+}
+
+static int _sccp_parse_xudts(struct msgb *msgb, struct sccp_parse_result *result)
+{
+	static const struct udt_offsets offsets = {
+		.header_size = sizeof(struct sccp_data_ext_unitdata_service),
+		.called_offset = offsetof(struct sccp_data_ext_unitdata_service, variable_called),
+		.calling_offset = offsetof(struct sccp_data_ext_unitdata_service, variable_calling),
+		.data_offset = offsetof(struct sccp_data_ext_unitdata_service, variable_data),
+	};
+
+	return _sccp_parse_unitdata(msgb, result, &offsets);
+}
+
 static int _sccp_parse_it(struct msgb *msgb, struct sccp_parse_result *result)
 {
 	static const uint32_t header_size = sizeof(struct sccp_data_it);
@@ -759,16 +840,15 @@ struct msgb *sccp_create_cr(const struct sccp_source_reference *src_ref,
 	struct msgb *request;
 	struct sccp_connection_request *req;
 	uint8_t *data;
-	uint8_t extra_size = 3 + 1;
 	int called_len;
 
-	if (l3_data && (l3_length < 3 || l3_length > 130)) {
-		LOGP(DSCCP, LOGL_ERROR, "Invalid amount of data... %zu\n", l3_length);
-		return NULL;
+	if (l3_data) {
+		if (l3_length < 3 || l3_length > SCCP_MAX_OPTIONAL_DATA) {
+			LOGP(DSCCP, LOGL_ERROR, "Invalid amount of data... %zu\n", l3_length);
+			return NULL;
+		}
 	}
 
-	if (l3_data)
-		extra_size += 2 + l3_length;
 	request = msgb_alloc_headroom(SCCP_MSG_SIZE,
 				      SCCP_MSG_HEADROOM, "sccp connection request");
 	request->l2h = &request->data[0];
@@ -1025,7 +1105,7 @@ static int _sccp_handle_connection_request(struct msgb *msgb, void *ctx)
 /* Handle the release confirmed */
 static int _sccp_handle_connection_release_complete(struct msgb *msgb)
 {
-	struct sccp_parse_result result;
+	struct sccp_parse_result result = {};
 	struct sccp_connection *conn;
 
 	if (_sccp_parse_connection_release_complete(msgb, &result) != 0)
@@ -1455,6 +1535,15 @@ int sccp_parse_header(struct msgb *msg, struct sccp_parse_result *result)
 	case SCCP_MSG_TYPE_UDT:
 		return _sccp_parse_udt(msg, result);
 		break;
+	case SCCP_MSG_TYPE_UDTS:
+		return _sccp_parse_udts(msg, result);
+		break;
+	case SCCP_MSG_TYPE_XUDT:
+		return _sccp_parse_xudt(msg, result);
+		break;
+	case SCCP_MSG_TYPE_XUDTS:
+		return _sccp_parse_xudts(msg, result);
+		break;
 	case SCCP_MSG_TYPE_IT:
 		return _sccp_parse_it(msg, result);
 		break;
