@@ -37,6 +37,8 @@
 #include <osmocom/sigtran/osmo_ss7.h>
 #include <osmocom/sigtran/protocol/mtp.h>
 
+#include <osmocom/sccp/sccp_types.h>
+
 #include "xua_internal.h"
 #include "sccp_internal.h"
 
@@ -148,9 +150,8 @@ DEFUN_ATTR(sccp_timer, sccp_timer_cmd,
 {
 	struct osmo_ss7_instance *ss7 = vty->index;
 	enum osmo_sccp_timer timer = get_string_value(osmo_sccp_timer_names, argv[0]);
-	struct osmo_sccp_timer_val set_val = { .s = atoi(argv[1]) };
 
-	if (timer < 0 || timer >= OSMO_SCCP_TIMERS_COUNT) {
+	if (timer <= 0 || timer >= OSMO_SCCP_TIMERS_LEN) {
 		vty_out(vty, "%% Invalid timer: %s%s", argv[0], VTY_NEWLINE);
 		return CMD_WARNING;
 	}
@@ -161,16 +162,39 @@ DEFUN_ATTR(sccp_timer, sccp_timer_cmd,
 		return CMD_WARNING;
 	}
 
-	ss7->sccp->timers[timer] = set_val;
+	OSMO_ASSERT(ss7->sccp->tdefs);
+	osmo_tdef_set(ss7->sccp->tdefs, timer, atoi(argv[1]), OSMO_TDEF_S);
 	return CMD_SUCCESS;
 }
 
-static const char *osmo_sccp_timer_val_name(const struct osmo_sccp_timer_val *val)
+DEFUN_ATTR(sccp_max_optional_data, sccp_max_optional_data_cmd,
+	   "sccp max-optional-data (<0-999999>|standard)",
+	   "Configure SCCP behavior\n"
+	   "Adjust the upper bound for the optional data length (the payload) for CR, CC, CREF and RLSD messages."
+	   " For any Optional Data part larger than this value in octets, send CR, CC, CREF and RLSD"
+	   " messages without any payload, and send the data payload in a separate Data Form 1 message."
+	   " ITU-T Q.713 sections 4.2 thru 4.5 define a limit of 130 bytes for the 'Data' parameter. This limit can be"
+	   " adjusted here. May be useful for interop with nonstandard SCCP peers.\n"
+	   "Set a non-standard maximum allowed number of bytes\n"
+	   "Use the ITU-T Q.713 4.2 to 4.5 standard value of 130\n",
+	   CMD_ATTR_IMMEDIATE)
 {
-	static char buf[16];
+	struct osmo_ss7_instance *ss7 = vty->index;
+	int val;
 
-	snprintf(buf, sizeof(buf), "%u", val->s);
-	return buf;
+	if (!strcmp(argv[0], "standard"))
+		val = -1;
+	else
+		val = atoi(argv[0]);
+
+	osmo_ss7_ensure_sccp(ss7);
+	if (!ss7->sccp) {
+		vty_out(vty, "%% Error: cannot instantiate SCCP instance%s", VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	osmo_sccp_set_max_optional_data(ss7->sccp, val);
+	return CMD_SUCCESS;
 }
 
 static void gen_sccp_timer_cmd_strs(struct cmd_element *cmd)
@@ -187,19 +211,19 @@ static void gen_sccp_timer_cmd_strs(struct cmd_element *cmd)
 			     "Configure SCCP timer values, see ITU-T Q.714\n");
 
 	for (i = 0; osmo_sccp_timer_names[i].str; i++) {
-		const struct osmo_sccp_timer_val *def;
+		const struct osmo_tdef *def;
 		enum osmo_sccp_timer timer;
 
 		timer = osmo_sccp_timer_names[i].value;
-		def = &osmo_sccp_timer_defaults[timer];
-		OSMO_ASSERT(timer >= 0 && timer < OSMO_SCCP_TIMERS_COUNT);
+		def = osmo_tdef_get_entry((struct osmo_tdef *)&osmo_sccp_timer_defaults, timer);
+		OSMO_ASSERT(def);
 
 		osmo_talloc_asprintf(tall_vty_ctx, cmd_str, "%s%s",
 				     i ? "|" : "",
-				     osmo_sccp_timer_name(timer));
-		osmo_talloc_asprintf(tall_vty_ctx, doc_str, "%s (default: %s)\n",
-				     osmo_sccp_timer_description(timer),
-				     osmo_sccp_timer_val_name(def));
+				     osmo_sccp_timer_names[i].str);
+		osmo_talloc_asprintf(tall_vty_ctx, doc_str, "%s (default: %lu)\n",
+				     def->desc,
+				     def->default_val);
 	}
 
 	osmo_talloc_asprintf(tall_vty_ctx, cmd_str, ") <1-999999>");
@@ -215,18 +239,22 @@ static void write_sccp_timers(struct vty *vty, const char *indent,
 {
 	int i;
 
-	for (i = 0; i < ARRAY_SIZE(inst->timers); i++) {
-		const struct osmo_sccp_timer_val *val = osmo_sccp_timer_get(inst, i, default_if_unset);
-		if (!val)
+	for (i = 0; osmo_sccp_timer_names[i].str; i++) {
+		const struct osmo_tdef *tdef = osmo_tdef_get_entry(inst->tdefs, osmo_sccp_timer_names[i].value);
+		if (!tdef)
 			continue;
-		vty_out(vty, "%ssccp-timer %s %s%s", indent, osmo_sccp_timer_name(i),
-			osmo_sccp_timer_val_name(val), VTY_NEWLINE);
+		if (!default_if_unset && tdef->val == tdef->default_val)
+			continue;
+		vty_out(vty, "%ssccp-timer %s %lu%s", indent, osmo_sccp_timer_names[i].str,
+			tdef->val, VTY_NEWLINE);
 	}
 }
 
 void osmo_sccp_vty_write_cs7_node(struct vty *vty, const char *indent, struct osmo_sccp_instance *inst)
 {
 	write_sccp_timers(vty, indent, inst, false);
+	if (inst->max_optional_data != SCCP_MAX_OPTIONAL_DATA)
+		vty_out(vty, "%ssccp max-optional-data %u%s", indent, inst->max_optional_data, VTY_NEWLINE);
 }
 
 DEFUN(show_sccp_timers, show_sccp_timers_cmd,
@@ -262,4 +290,5 @@ void osmo_sccp_vty_init(void)
 	install_lib_element_ve(&show_sccp_timers_cmd);
 	gen_sccp_timer_cmd_strs(&sccp_timer_cmd);
 	install_lib_element(L_CS7_NODE, &sccp_timer_cmd);
+	install_lib_element(L_CS7_NODE, &sccp_max_optional_data_cmd);
 }
