@@ -180,13 +180,15 @@ static const uint16_t coit_mand_ies[] = {
 	SUA_IEI_ROUTE_CTX, SUA_IEI_PROTO_CLASS, SUA_IEI_SRC_REF,
 	SUA_IEI_DEST_REF, 0
 };
+
+/* ITU-T Rec Q.713 */
 static const struct value_string sua_co_msgt_names[] = {
 	{ SUA_CO_CODT,		"CODT" },
 	{ SUA_CO_CODA,		"CODA" },
-	{ SUA_CO_CORE,		"CORE" },
-	{ SUA_CO_COAK,		"COAK" },
-	{ SUA_CO_COREF,		"COREF" },
-	{ SUA_CO_RELRE,		"RELRE" },
+	{ SUA_CO_CORE,		"CORE" },  /* §4.2 Connection request (CR) */
+	{ SUA_CO_COAK,		"COAK" },  /* §4.3 Connection confirm (CC) */
+	{ SUA_CO_COREF,		"COREF" }, /* §4.4 Connection refused (CREF) */
+	{ SUA_CO_RELRE,		"RELRE" }, /* §4.5 Released (RLSD) */
 	{ SUA_CO_RELCO,		"RELCO" },
 	{ SUA_CO_RESRE,		"RESRE" },
 	{ SUA_CO_RESCO,		"RESCO" },
@@ -258,7 +260,7 @@ static struct xua_msg *sua_gen_error_msg(uint32_t err_code, struct msgb *msg)
 }
 
 /***********************************************************************
- * Transmitting SUA messsages to SCTP
+ * Transmitting SUA messages to SCTP
  ***********************************************************************/
 
 static struct msgb *sua_to_msg(struct xua_msg *xua)
@@ -316,7 +318,7 @@ int sua_tx_xua_as(struct osmo_ss7_as *as, struct xua_msg *xua)
 		return -1;
 
 	/* send the msg to the AS for transmission.  The AS FSM might
-	 * (depending on its state) enqueue it before trnsmission */
+	 * (depending on its state) enqueue it before transmission */
 	rc = osmo_fsm_inst_dispatch(as->fi, XUA_AS_E_TRANSFER_REQ, msg);
 	if (rc < 0)
 		msgb_free(msg);
@@ -324,7 +326,7 @@ int sua_tx_xua_as(struct osmo_ss7_as *as, struct xua_msg *xua)
 }
 
 /***********************************************************************
- * Receiving SUA messsages from SCTP
+ * Receiving SUA messages from SCTP
  ***********************************************************************/
 
 /*! \brief Decode SUA Global Title according to RFC3868 Section 3.10.2.3
@@ -427,7 +429,7 @@ int sua_addr_parse_part(struct osmo_sccp_addr *out,
 	 * probably want to have a separate general parsing function storing
 	 * the subparts in xua_msg_part. But before we do, we should find more
 	 * users of this subpart parsing and be aware of the performance
-	 * tradeoff.
+	 * trade-off.
 	 */
 
 	while (pos + sizeof(*par) < param->len) {
@@ -619,7 +621,7 @@ static int sua_rx_mgmt_ntfy(struct osmo_ss7_asp *asp, struct xua_msg *xua)
 	if (ntfy.info_string)
 		talloc_free(ntfy.info_string);
 
-	/* TODO: should we report this soemwhere? */
+	/* TODO: should we report this somewhere? */
 	return 0;
 }
 
@@ -680,7 +682,7 @@ int sua_rx_msg(struct osmo_ss7_asp *asp, struct msgb *msg)
 	OSMO_ASSERT(asp->cfg.proto == OSMO_SS7_ASP_PROT_SUA);
 
 	/* caller owns msg memory, we shall neither free it here nor
-	 * keep references beyon the execution of this function and its
+	 * keep references beyond the execution of this function and its
 	 * callees. */
 
 	if (!asp->inst->sccp) {
@@ -783,7 +785,7 @@ static struct xua_msg *sua_encode_duna(const uint32_t *rctx, unsigned int num_rc
 
 	xua->hdr = XUA_HDR(SUA_MSGC_SNM, SUA_SNM_DUNA);
 	xua->hdr.version = SUA_VERSION;
-	if (rctx)
+	if (rctx && num_rctx)
 		xua_msg_add_data(xua, SUA_IEI_ROUTE_CTX, num_rctx * 4, (const uint8_t *)rctx);
 
 	xua_msg_add_data(xua, SUA_IEI_AFFECTED_PC, num_aff_pc * 4, (const uint8_t *) aff_pc);
@@ -838,7 +840,7 @@ static struct xua_msg *sua_encode_dupu(const uint32_t *rctx, unsigned int num_rc
 
 	xua->hdr = XUA_HDR(SUA_MSGC_SNM, SUA_SNM_DUNA);
 	xua->hdr.version = SUA_VERSION;
-	if (rctx)
+	if (rctx && num_rctx)
 		xua_msg_add_data(xua, SUA_IEI_ROUTE_CTX, num_rctx * sizeof(*rctx), (const uint8_t *)rctx);
 
 	xua_msg_add_u32(xua, SUA_IEI_AFFECTED_PC, dpc);
@@ -893,7 +895,7 @@ void sua_tx_snm_congestion(struct osmo_ss7_asp *asp, const uint32_t *rctx, unsig
 
 	xua->hdr = XUA_HDR(SUA_MSGC_SNM, SUA_SNM_SCON);
 	xua->hdr.version = SUA_VERSION;
-	if (rctx)
+	if (rctx && num_rctx)
 		xua_msg_add_data(xua, SUA_IEI_ROUTE_CTX, num_rctx * sizeof(*rctx), (const uint8_t *)rctx);
 	xua_msg_add_data(xua, SUA_IEI_AFFECTED_PC, num_aff_pc * sizeof(*aff_pc), (const uint8_t *) aff_pc);
 	if (ssn)
