@@ -32,6 +32,7 @@
 #include <osmocom/core/utils.h>
 #include <osmocom/core/logging.h>
 #include <osmocom/core/application.h>
+#include <osmocom/core/rate_ctr.h>
 #include <osmocom/core/fsm.h>
 #include <osmocom/vty/vty.h>
 #include <osmocom/vty/stats.h>
@@ -40,6 +41,7 @@
 #include <osmocom/vty/telnet_interface.h>
 #include <osmocom/vty/logging.h>
 #include <osmocom/vty/misc.h>
+#include <osmocom/vty/cpu_sched_vty.h>
 
 #include <osmocom/sigtran/osmo_ss7.h>
 #include <osmocom/sigtran/sccp_sap.h>
@@ -59,10 +61,10 @@ static const struct log_info log_info = {
 };
 
 static const char stp_copyright[] =
-	"Copyright (C) 2015-2017 by Harald Welte <laforge@gnumonks.org>\r\n"
-	"Contributions by Holger Freyther, Neels Hofmeyr\r\n"
+	"Copyright (C) 2015-2020 by Harald Welte <laforge@gnumonks.org>\r\n"
+	"Contributions by Holger Freyther, Neels Hofmeyr, Pau Espin, Vadim Yanitskiy\r\n"
 	"License GPLv2+: GNU GPL Version 2 or later <http://gnu.org/licenses/gpl-2.0.html>\r\n"
-	"This is free software: you are free ot change and redistribute it.\r\n"
+	"This is free software: you are free to change and redistribute it.\r\n"
 	"There is NO WARRANTY, to the extent permitted by law.\r\n\r\n"
 	"Free Software lives by contribution.  If you use this, please contribute!\r\n";
 
@@ -88,17 +90,49 @@ static void print_help(void)
 	printf("  -D --daemonize		Fork the process into a background daemon\n");
 	printf("  -c --config-file filename	The config file to use. Default: ./osmo-stp.cfg\n");
 	printf("  -V --version			Print the version of OsmoSTP\n");
+
+	printf("\nVTY reference generation:\n");
+	printf("     --vty-ref-mode MODE        VTY reference generation mode (e.g. 'expert').\n");
+	printf("     --vty-ref-xml              Generate the VTY reference XML output and exit.\n");
+}
+
+static void handle_long_options(const char *prog_name, const int long_option)
+{
+	static int vty_ref_mode = VTY_REF_GEN_MODE_DEFAULT;
+
+	switch (long_option) {
+	case 1:
+		vty_ref_mode = get_string_value(vty_ref_gen_mode_names, optarg);
+		if (vty_ref_mode < 0) {
+			fprintf(stderr, "%s: Unknown VTY reference generation "
+				"mode '%s'\n", prog_name, optarg);
+			exit(2);
+		}
+		break;
+	case 2:
+		fprintf(stderr, "Generating the VTY reference in mode '%s' (%s)\n",
+			get_value_string(vty_ref_gen_mode_names, vty_ref_mode),
+			get_value_string(vty_ref_gen_mode_desc, vty_ref_mode));
+		vty_dump_xml_ref_mode(stdout, (enum vty_ref_gen_mode) vty_ref_mode);
+		exit(0);
+	default:
+		fprintf(stderr, "%s: error parsing cmdline options\n", prog_name);
+		exit(2);
+	}
 }
 
 static void handle_options(int argc, char **argv)
 {
 	while (1) {
 		int option_index = 0, c;
+		static int long_option = 0;
 		static const struct option long_options[] = {
 			{ "help", 0, 0, 'h' },
 			{ "daemonize", 0, 0, 'D' },
 			{ "config-file", 1, 0, 'c' },
 			{ "version", 0, 0, 'V' },
+			{ "vty-ref-mode", 1, &long_option, 1 },
+			{ "vty-ref-xml", 0, &long_option, 2 },
 			{ NULL, 0, 0, 0 }
 		};
 
@@ -107,6 +141,9 @@ static void handle_options(int argc, char **argv)
 			break;
 
 		switch (c) {
+		case 0:
+			handle_long_options(argv[0], long_option);
+			break;
 		case 'h':
 			print_help();
 			exit(0);
@@ -134,11 +171,11 @@ static void handle_options(int argc, char **argv)
 	}
 }
 
-static void signal_handler(int signal)
+static void signal_handler(int signum)
 {
-	fprintf(stdout, "signal %u received\n", signal);
+	fprintf(stdout, "signal %u received\n", signum);
 
-	switch (signal) {
+	switch (signum) {
 	case SIGINT:
 	case SIGTERM:
 		/* FIXME: handle the signal somewhere else and gracefully shut down
@@ -149,8 +186,17 @@ static void signal_handler(int signal)
 		break;
 	case SIGABRT:
 		osmo_generate_backtrace();
-		/* in case of abort, we want to obtain a talloc report
-		 * and then return to the caller, who will abort the process */
+		/* in case of abort, we want to obtain a talloc report and
+		 * then run default SIGABRT handler, who will generate coredump
+		 * and abort the process. abort() should do this for us after we
+		 * return, but program wouldn't exit if an external SIGABRT is
+		 * received.
+		 */
+		talloc_report(tall_vty_ctx, stderr);
+		talloc_report_full(tall_stp_ctx, stderr);
+		signal(SIGABRT, SIG_DFL);
+		raise(SIGABRT);
+		break;
 	case SIGUSR1:
 		talloc_report(tall_vty_ctx, stderr);
 		talloc_report_full(tall_stp_ctx, stderr);
@@ -167,6 +213,8 @@ int main(int argc, char **argv)
 {
 	int rc;
 
+	srand(time(NULL));
+
 	tall_stp_ctx = talloc_named_const(NULL, 1, "osmo-stp");
 	msgb_talloc_ctx_init(tall_stp_ctx, 0);
 	osmo_init_logging2(tall_stp_ctx, &log_info);
@@ -175,20 +223,21 @@ int main(int argc, char **argv)
 	vty_info.tall_ctx = tall_stp_ctx;
 	vty_init(&vty_info);
 
-	handle_options(argc, argv);
-
-	fputs(stp_copyright, stdout);
-	fputs("\n", stdout);
-
 	OSMO_ASSERT(osmo_ss7_init() == 0);
 	osmo_fsm_log_addr(false);
 	logging_vty_add_cmds();
 	osmo_stats_vty_add_cmds();
 	osmo_ss7_vty_init_sg(tall_stp_ctx);
 	osmo_sccp_vty_init();
+	osmo_cpu_sched_vty_init(tall_stp_ctx);
 	osmo_fsm_vty_add_cmds();
 	osmo_talloc_vty_add_cmds();
 
+	handle_options(argc, argv);
+
+	fputs(stp_copyright, stdout);
+	fputs("\n", stdout);
+
 	rc = vty_read_config_file(cmdline_config.config_file, NULL);
 	if (rc < 0) {
 		fprintf(stderr, "Failed to parse the config file '%s'\n",
@@ -196,7 +245,7 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	rc = telnet_init_dynif(tall_stp_ctx, NULL, vty_get_bind_addr(), OSMO_VTY_PORT_STP);
+	rc = telnet_init_default(tall_stp_ctx, NULL, OSMO_VTY_PORT_STP);
 	if (rc < 0) {
 		perror("Error binding VTY port\n");
 		exit(1);
@@ -208,6 +257,7 @@ int main(int argc, char **argv)
 	signal(SIGUSR1, &signal_handler);
 	signal(SIGUSR2, &signal_handler);
 	osmo_init_ignore_signals();
+	rate_ctr_init(tall_stp_ctx);
 
 	if (cmdline_config.daemonize) {
 		rc = osmo_daemonize();
